import requests
from kopeechka.types_kopeechka import MailboxCancel

def mailbox_cancel(api_token: str, id: int) -> MailboxCancel:
    query = {
        "id": id,
        "token": api_token,
        "type": "json",
        "api": 2.0
    }
    response = requests.get(url="http://api.kopeechka.store/mailbox-cancel", params=query)
    return MailboxCancel(response.json())


import requests
from kopeechka.types_kopeechka import MailboxGetMessage

def mailbox_get_message(api_token: str, full: int, id: int) -> MailboxGetMessage:
    query = {
        "full": full,
        "id": id,
        "token": api_token,
        "type": "json",
        "api": 2.0
    }

    response = requests.get(url="http://api.kopeechka.store/mailbox-get-message", params=query)
    return MailboxGetMessage(response.json())


from kopeechka.errors import BAD_TOKEN, BAD_SITE, BAD_DOMAIN, BAD_BALANCE, SYSTEM_ERROR, OUT_OF_STOCK, TIME_LIMIT_EXCEED, UNKNOWN_ERROR

class MailboxGetEmail():
    def __init__(self, json: dict):
        self.data: dict = json
        self.status: str = self.data.get("status")
        self.id: str | None = self.data.get("id")
        self.mail: str | None = self.data.get("mail")
        self.value: str | None = self.data.get("value")
        if self.status == "ERROR":
            if self.value == "BAD_TOKEN":
                raise BAD_TOKEN(data=self.data)
            elif self.value == "BAD_SITE":
                raise BAD_SITE(data=self.data)
            elif self.value == "BAD_DOMAIN":
                raise BAD_DOMAIN(data=self.data)
            elif self.value == "BAD_BALANCE":
                raise BAD_BALANCE(data=self.data)
            elif self.value == "OUT_OF_STOCK":
                raise OUT_OF_STOCK(data=self.data)
            elif self.value == "SYSTEM_ERROR":
                raise SYSTEM_ERROR(data=self.data)
            elif self.value == "TIME_LIMIT_EXCEED":
                raise TIME_LIMIT_EXCEED(data=self.data)



from .bad_token import BAD_TOKEN
from .bad_balance import BAD_BALANCE
from .bad_site import BAD_SITE
from .bad_domain import BAD_DOMAIN
from .out_of_stock import OUT_OF_STOCK
from .system_error import SYSTEM_ERROR
from .time_limit_exceed import TIME_LIMIT_EXCEED
from .activation_canceled import ACTIVATION_CANCELED
from .no_activation import NO_ACTIVATION
from .wait_link import WAIT_LINK
from .activation_not_found import ACTIVATION_NOT_FOUND
from .bad_email import BAD_EMAIL
from .bad_comment import BAD_COMMENT
from .unknown_error import UNKNOWN_ERROR


from .user_balance import user_balance
from .mailbox_get_message import mailbox_get_message
from .mailbox_get_email import mailbox_get_email
from .mailbox_cancel import mailbox_cancel
from .mailbox_reorder import mailbox_reorder
from .mailbox_get_fresh_id import mailbox_get_fresh_id
from .mailbox_set_comment import mailbox_set_comment
from .mailbox_get_bulk import mailbox_get_bulk
from .mailbox_get_domains import mailbox_get_domains
from .mailbox_zones import mailbox_zones



from .zone import Zone
from .popular import Popular
from kopeechka.errors import UNKNOWN_ERROR

class MailboxZones():
    def __init__(self, json: dict):
        self.data: dict = json
        self.status: str = self.data.get("status")
        if self.status == "ERROR":
            self.value: str = self.data.get("value")
            raise UNKNOWN_ERROR(text=self.value, data=self.data)
        if self.data.get("zones"):
            self.zones = self.data.get("zones")
        else:
            self.zones = []
        self.zones: list[Zone] = [Zone(data) for data in self.zones]
        if self.data.get("popular"):
            self.popular = self.data.get("popular")
        else:
            self.popular = []
        self.popular: list[Popular] = [Popular(data) for data in self.popular]




import requests
from kopeechka.types_kopeechka import MailboxGetDomains

def mailbox_get_domains(api_token: str, site: str) -> MailboxGetDomains:
    query = {
        "token": api_token,
        "site": site,
        "type": "json",
        "api": 2.0
    }
    response = requests.get("http://api.kopeechka.store/mailbox-get-domains", params=query)
    return MailboxGetDomains(response.json())


from kopeechka.utils import request_get
from kopeechka.types_kopeechka import MailboxZones

async def mailbox_zones(popular: int, zones: int) -> MailboxZones:
    query = {
        "popular": popular,
        "zones": zones
    }
    return await request_get(url="https://api.kopeechka.store/mailbox-zones", params=query, type_kopeechka=MailboxZones)



class BAD_BALANCE(Exception):
    def __init__(self, text="There are not enough funds to perform the operation.", data={}):
        self.text = text
        self.data = data

    def __str__(self):
        return self.text


class BAD_SITE(Exception):
    def __init__(self, text="You specified the site incorrectly.", data={}):
        self.text = text
        self.data = data

    def __str__(self):
        return self.text


from kopeechka.utils import request_get
from kopeechka.types_kopeechka import MailboxGetBulk

async def mailbox_get_bulk(api_token: str, count: int, comment: str, email: str, site: str) -> MailboxGetBulk:
    query = {
        "token": api_token,
        "count": count,
        "comment": comment,
        "email": email,
        "site": site,
        "type": "json",
        "api": 2.0
    }
    return await request_get("http://api.kopeechka.store/mailbox-get-bulk", params=query, type_kopeechka=MailboxGetBulk)


from kopeechka.utils import request_get
from kopeechka.types_kopeechka import MailboxGetFreshId

async def mailbox_get_fresh_id(api_token: str, site: str, email: str) -> MailboxGetFreshId:
    query = {
        "token": api_token,
        "site": site,
        "email": email,
        "type": "json",
        "api": 2.0
    }
    return await request_get(url="http://api.kopeechka.store/mailbox-get-fresh-id", params=query, type_kopeechka=MailboxGetFreshId)


class TIME_LIMIT_EXCEED(Exception):
    def __init__(self, text="The limit of mail orders per second has been reached (applies to special tariffs), it is necessary to expand the tariff.", data={}):
        self.text = text
        self.data = data

    def __str__(self):
        return self.text


import requests
from kopeechka.types_kopeechka import MailboxZones

def mailbox_zones(popular: int, zones: int) -> MailboxZones:
    query = {
        "popular": popular,
        "zones": zones
    }
    response = requests.get("https://api.kopeechka.store/mailbox-zones", params=query)
    return MailboxZones(response.json())



from kopeechka.utils import request_get
from kopeechka.types_kopeechka import MailboxSetComment

async def mailbox_set_comment(api_token: str, id: int, comment: str) -> MailboxSetComment:
    query = {
        "token": api_token,
        "id": id,
        "comment": comment,
        "type": "json",
        "api": 2.0
    }
    return await request_get(url='http://api.kopeechka.store/mailbox-set-comment', params=query, type_kopeechka=MailboxSetComment)


from kopeechka.utils import request_get
from kopeechka.types_kopeechka import MailboxReorder

async def mailbox_reorder(api_token: str, site: str, email: str, regex: str, subject: str) -> MailboxReorder:
    query = {
        "site": site,
        "email": email,
        "regex": regex,
        "token": api_token,
        "type": "json",
        "subject": subject,
        "api": 2.0
    }
    return await request_get(url="http://api.kopeechka.store/mailbox-reorder", params=query, type_kopeechka=MailboxReorder)



import requests
from kopeechka.types_kopeechka import MailboxSetComment

def mailbox_set_comment(api_token: str, id: int, comment: str) -> MailboxSetComment:
    query = {
        "token": api_token,
        "id": id,
        "comment": comment,
        "type": "json",
        "api": 2.0
    }
    response = requests.get('http://api.kopeechka.store/mailbox-set-comment', params=query)
    return MailboxSetComment(response.json())


from kopeechka.utils import request_get
from kopeechka.types_kopeechka import MailboxGetEmail

async def mailbox_get_email(api_token: str, site: str, mail_type: str, sender: str, regex: str, soft_id: int, investor: int, subject: str, clear: int) -> MailboxGetEmail:
    query = {
        "site": site,
        "mail_type": mail_type,
        "token": api_token,
        "sender": sender,
        "regex": regex,
        "soft_id": soft_id,
        "investor": investor,
        "subject": subject,
        "clear": clear,
        "type": "json",
        "api": 2.0
    }
    return await request_get(url="http://api.kopeechka.store/mailbox-get-email", params=query, type_kopeechka=MailboxGetEmail)


from .email import Email
from kopeechka.errors import BAD_TOKEN, UNKNOWN_ERROR

class MailboxGetBulk():
    def __init__(self, json: dict):
        self.data: dict = json
        self.status: str = self.data.get("status")
        self.count: int | None = self.data.get("count")
        self.items: list | None = self.data.get("items")
        self.value: str | None = self.data.get("value")
        if self.status == "ERROR":
            if self.value == "BAD_TOKEN":
                raise BAD_TOKEN(data=self.data)
            else:
                raise UNKNOWN_ERROR(text=self.value, data=self.data)
        if self.items:
            self.items = [Email(data) for data in self.items]
        else:
            self.items = []


import requests
from kopeechka.types_kopeechka import MailboxReorder

def mailbox_reorder(api_token: str, site: str, email: str, regex: str, subject: str) -> MailboxReorder:
    query = {
        "site": site,
        "email": email,
        "regex": regex,
        "token": api_token,
        "type": "json",
        "subject": subject,
        "api": 2.0
    }
    response = requests.get(url="http://api.kopeechka.store/mailbox-reorder", params=query)
    return MailboxReorder(response.json())



from .userBalance import UserBalance
from .mailboxGetEmail import MailboxGetEmail
from .mailboxGetMessage import MailboxGetMessage
from .mailboxCancel import MailboxCancel
from .mailboxGetFreshId import MailboxGetFreshId
from .email import Email
from .mailboxGetBulk import MailboxGetBulk
from .mailboxReorder import MailboxReorder
from .zone import Zone
from .mailboxZones import MailboxZones
from .mailboxGetDomains import MailboxGetDomains
from .mailboxSetComment import MailboxSetComment
from .popular import Popular


class Email():
    def __init__(self, json: dict):
        self.data: dict = json
        self.id: int = self.data.get("id")
        self.service: str = self.data.get("service")
        self.email: str = self.data.get("email")
        self.date: int = self.data.get("date")
        self.status: str = self.data.get("status")
        self.value: str = self.data.get("value")
        self.comment: str = self.data.get("comment")



from kopeechka.utils import request_get
from kopeechka.types_kopeechka import UserBalance

async def user_balance(api_token: str) -> UserBalance:
    query = {
        "token": api_token,
        "type": "json",
        "api": 2.0
    }
    return await request_get(url="https://api.kopeechka.store/user-balance", params=query, type_kopeechka=UserBalance)



class Popular():
    def __init__(self, json: dict):
        self.data: dict = json
        self.name: str = self.data.get("name")
        self.cost: int = self.data.get("cost")
        self.count: int = self.data.get("count")


class BAD_DOMAIN(Exception):
    def __init__(self, text="We do not have such a domain/domain zone.", data={}):
        self.text = text
        self.data = data

    def __str__(self):
        return self.text


class BAD_COMMENT(Exception):
    def __init__(self, text="Activattion not found (It`s not your activation).", data={}):
        self.text = text
        self.data = data

    def __str__(self):
        return self.text


class OUT_OF_STOCK(Exception):
    def __init__(self, text="There is no mail with such settings. Try changing $MAIL_TYPE or write to support - we'll try to add mailboxes.", data={}):
        self.text = text
        self.data = data

    def __str__(self):
        return self.text


import aiohttp

async def request_get(url: str, params: dict, type_kopeechka):
    async with aiohttp.ClientSession() as session:
        for key in {key:params[key] for key in params}:
            if not params.get(key):
                params.pop(key)
        async with session.get(url=url, params=params) as response:
            return type_kopeechka(await response.json())


class ACTIVATION_NOT_FOUND(Exception):
    def __init__(self, text="First letter has not been received, reorder isn`t possible.", data={}):
        self.text = text
        self.data = data

    def __str__(self):
        return self.text